#include <EEPROM.h>
#define   DIG0    2
#define   DIG1    3
#define   DIG2    4
#define   DIG3    5
#define   DIG4    6
#define   RESET   7
#define   BUTTON  8

uint32_t startcount;
uint32_t firstemptydword;
  
void setup() 
{
  uint16_t address;
  uint8_t eb_1, eb_2, eb_3, eb_4;
  uint8_t eepromisfull;
  uint32_t eepdword;

  Serial.begin(115200);

  pinMode(BUTTON, INPUT);         // set pin as input

  pinMode(DIG0, OUTPUT);          // set pin to output
  pinMode(DIG1, OUTPUT);          // set pin to output
  pinMode(DIG2, OUTPUT);          // set pin to output
  pinMode(DIG3, OUTPUT);          // set pin to output
  pinMode(DIG4, OUTPUT);          // set pin to output
  pinMode(RESET, OUTPUT);         // set RESET pin to output

  digitalWrite(RESET, HIGH);      // reset counter to have an init point
  digitalWrite(DIG0, HIGH);
  digitalWrite(DIG1, HIGH);
  digitalWrite(DIG2, HIGH);
  digitalWrite(DIG3, HIGH);
  digitalWrite(DIG4, HIGH);

//  // EEPROM ERASER
//  for(address = 0; address < EEPROM.length(); address ++)
//  {
//    EEPROM.write(address, 0);
//  }

//  // EEPROM DEBUGGER
//  for(address = 0; address < EEPROM.length(); address += 4)
//  {
//    eb_4 = EEPROM.read(address);  
//    eb_3 = EEPROM.read(address + 1);
//    eb_2 = EEPROM.read(address + 2);
//    eb_1 = EEPROM.read(address + 3);
//    eepdword = ((uint32_t)eb_4);
//    eepdword = (eepdword << 8) | eb_3;
//    eepdword = (eepdword << 8) | eb_2;
//    eepdword = (eepdword << 8) | eb_1;
//
//    Serial.print("ADDRESS: ");
//    Serial.print(address);
//    Serial.print(" DATA: ");
//    Serial.println(eepdword);
//  }
  
  eepromisfull = 1;
  
  for(address = 0; address < EEPROM.length(); address += 4)
  {
    eb_4 = EEPROM.read(address);  
    eb_3 = EEPROM.read(address + 1);
    eb_2 = EEPROM.read(address + 2);
    eb_1 = EEPROM.read(address + 3);
    eepdword = ((uint32_t)eb_4);
    eepdword = (eepdword << 8) | eb_3;
    eepdword = (eepdword << 8) | eb_2;
    eepdword = (eepdword << 8) | eb_1;
    
    startcount = eepdword;        // save start count value
    
    if(eepdword == 0x00000000)    // if 4 consecutive bytes are empty
    {
      firstemptydword = address;  // save address as being first empty location
      eepromisfull = 0;           // eeprom is not full, no erase needed

      address -= 4;               // go back to last location with value and read
      eb_4 = EEPROM.read(address);  
      eb_3 = EEPROM.read(address + 1);
      eb_2 = EEPROM.read(address + 2);
      eb_1 = EEPROM.read(address + 3);
      eepdword = ((uint32_t)eb_4);
      eepdword = (eepdword << 8) | eb_3;
      eepdword = (eepdword << 8) | eb_2;
      eepdword = (eepdword << 8) | eb_1;
      
      startcount = eepdword;      // get start count value
      break;                      // quit looping
    }  
  }
  
  if(eepromisfull == 1)
  {
    for(address = 0; address < EEPROM.length(); address ++)
    {
      EEPROM.write(address, 0);
    }
    firstemptydword = 0;
    eepromisfull = 0;  
  }

  // startcount = 23455;
  CNT_SetStartValue(startcount);
}

void loop() 
{
  while(digitalRead(BUTTON) == 1);
  while(digitalRead(BUTTON) == 0);
  CNT_AdvanceCounter();
  delay(100);
}

void CNT_AdvanceCounter( void )
{
  startcount++;
  CNT_SaveCountToEEPROM();
    
  if(startcount%10000 == 0)
  {
    digitalWrite(DIG4, LOW);
    delay(20);
    digitalWrite(DIG4, HIGH);  
  }
  
  if(startcount%1000 == 0)
  {
    digitalWrite(DIG3, LOW);
    delay(20);
    digitalWrite(DIG3, HIGH);  
  }
    
  if(startcount%100 == 0)
  {
    digitalWrite(DIG2, LOW);
    delay(20);
    digitalWrite(DIG2, HIGH);  
  }
  
  if(startcount%10 == 0)
  {
    digitalWrite(DIG1, LOW);
    delay(20);
    digitalWrite(DIG1, HIGH);  
  }
  
  digitalWrite(DIG0, LOW);
  delay(20);
  digitalWrite(DIG0, HIGH);

  delay(10);  
}

void CNT_SetStartValue( uint32_t number )
{
  uint8_t digit, iterator;
  
  digit = number%10;
  number /= 10;
  for(iterator = 0; iterator < digit; iterator++)
  {
    digitalWrite(DIG0, LOW);
    delay(20);
    digitalWrite(DIG0, HIGH);
    delay(20);
  }
  
  digit = number%10;
  number /= 10;
  for(iterator = 0; iterator < digit; iterator++)
  {
    digitalWrite(DIG1, LOW);
    delay(20);
    digitalWrite(DIG1, HIGH);
    delay(20);  
  }
  
  digit = number%10;
  number /= 10;
  for(iterator = 0; iterator < digit; iterator++)
  {
    digitalWrite(DIG2, LOW);
    delay(20);
    digitalWrite(DIG2, HIGH);  
    delay(20);
  }

  digit = number%10;
  number /= 10;
  for(iterator = 0; iterator < digit; iterator++)
  {
    digitalWrite(DIG3, LOW);
    delay(20);
    digitalWrite(DIG3, HIGH);
    delay(20);  
  }

  digit = number%10;
  for(iterator = 0; iterator < digit; iterator++)
  {
    digitalWrite(DIG4, LOW);
    delay(20);
    digitalWrite(DIG4, HIGH);  
    delay(20);
  }

  //displaynumber(count);
  delay(10);
}

void CNT_SaveCountToEEPROM( void )
{
  uint8_t eb_1, eb_2, eb_3, eb_4;
  uint32_t startcount_save;

  startcount_save = startcount;
  
  eb_1 = (uint8_t)(startcount & 0x000000FF); startcount = startcount >> 8;
  eb_2 = (uint8_t)(startcount & 0x000000FF); startcount = startcount >> 8;
  eb_3 = (uint8_t)(startcount & 0x000000FF); startcount = startcount >> 8;
  eb_4 = (uint8_t)(startcount & 0x000000FF); startcount = startcount >> 8;
  startcount = startcount_save;

  Serial.print("SAVING CNT TO LOCATION ");
  Serial.print(firstemptydword);
  Serial.print(", COUNT IS: ");
  Serial.print(startcount_save);
  
  EEPROM.write(firstemptydword, eb_4);
  EEPROM.write(firstemptydword + 1, eb_3);
  EEPROM.write(firstemptydword + 2, eb_2);
  EEPROM.write(firstemptydword + 3, eb_1); 
  
  if(firstemptydword < EEPROM.length())
  {
    firstemptydword += 4; 
  }
  else
  {
    firstemptydword = 0;
  }
  
  Serial.println(". SAVE OK.");
}
